package ru.octol1ttle.knockdowns.client;

import net.minecraft.client.Minecraft;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraftforge.fml.common.event.FMLInitializationEvent;
import net.minecraftforge.fml.common.network.simpleimpl.IMessage;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;
import ru.octol1ttle.knockdowns.client.communication.CalloutManager;
import ru.octol1ttle.knockdowns.client.communication.KnockedNotificationManager;
import ru.octol1ttle.knockdowns.client.event.KnockdownsKeyListener;
import ru.octol1ttle.knockdowns.client.util.DirectionalCallSound;
import ru.octol1ttle.knockdowns.common.IClientProxy;
import ru.octol1ttle.knockdowns.common.data.IKnockdownsPlayerData;
import ru.octol1ttle.knockdowns.common.network.packets.s2c.PlayerCalloutS2CPacket;
import ru.octol1ttle.knockdowns.common.network.packets.s2c.PlayerKnockedDownS2CPacket;
import ru.octol1ttle.knockdowns.common.network.packets.s2c.SynchronizePlayerDataS2CPacket;
import ru.octol1ttle.knockdowns.common.network.packets.s2c.SynchronizeReviversS2CPacket;
import ru.octol1ttle.knockdowns.common.registry.KnockdownsSoundEvents;

import static ru.octol1ttle.knockdowns.common.KnockdownsUtils.INITIAL_REVIVE_TIME_LEFT;

@SideOnly(Side.CLIENT)
public class ClientProxy implements IClientProxy {
    private static final Minecraft client = Minecraft.getMinecraft();

    @Override
    public void onFMLInit(FMLInitializationEvent event) {
        KnockdownsKeyListener.registerKeyBindings();
    }

    @Override
    public <T> T handleMessage(IMessage message) {
        if (message instanceof PlayerCalloutS2CPacket) {
            PlayerCalloutS2CPacket packet = (PlayerCalloutS2CPacket) message;
            client.addScheduledTask(() -> {
                if (CalloutManager.addOrUpdateCallout(packet)) {
                    CalloutManager.playCalloutSound(packet);
                }
            });
        } else if (message instanceof PlayerKnockedDownS2CPacket) {
            PlayerKnockedDownS2CPacket packet = (PlayerKnockedDownS2CPacket) message;
            client.addScheduledTask(() -> {
                EntityPlayer entity = (EntityPlayer) client.world.getEntityByID(packet.playerId);
                if (entity != null) {
                    IKnockdownsPlayerData data = IKnockdownsPlayerData.get(entity);
                    data.setKnockedDown(true);
                    data.setReviveTimeLeft(INITIAL_REVIVE_TIME_LEFT);
                    data.getRevivers().clear();
                }

                if (client.player.dimension == packet.dimensionId) {
                    client.getSoundHandler().playSound(new DirectionalCallSound(KnockdownsSoundEvents.KNOCKED_DOWN, entity, packet.position));
                    KnockedNotificationManager.addKnockedNotification(packet.playerId, packet.position);
                }
            });
        } else if (message instanceof SynchronizePlayerDataS2CPacket.KnockedDown) {
            SynchronizePlayerDataS2CPacket.KnockedDown packet = (SynchronizePlayerDataS2CPacket.KnockedDown) message;
            client.addScheduledTask(() -> {
                EntityPlayer entity = (EntityPlayer) client.world.getEntityByID(packet.playerId);
                if (entity != null) {
                    IKnockdownsPlayerData data = IKnockdownsPlayerData.get(entity);
                    data.setKnockedDown(packet.knockedDown);
                    data.setReviveTimeLeft(INITIAL_REVIVE_TIME_LEFT);
                    data.getRevivers().clear();
                }
            });
        } else if (message instanceof SynchronizePlayerDataS2CPacket.ReviveTimeLeft) {
            SynchronizePlayerDataS2CPacket.ReviveTimeLeft packet = (SynchronizePlayerDataS2CPacket.ReviveTimeLeft) message;
            client.addScheduledTask(() -> {
                EntityPlayer entity = (EntityPlayer) client.world.getEntityByID(packet.playerId);
                if (entity != null) {
                    IKnockdownsPlayerData data = IKnockdownsPlayerData.get(entity);
                    data.setReviveTimeLeft(packet.reviveTimeLeft);
                }
            });
        } else if (message instanceof SynchronizePlayerDataS2CPacket.Full) {
            SynchronizePlayerDataS2CPacket.Full packet = (SynchronizePlayerDataS2CPacket.Full) message;
            client.addScheduledTask(() -> {
                EntityPlayer entity = (EntityPlayer) client.world.getEntityByID(packet.playerId);
                if (entity != null) {
                    IKnockdownsPlayerData data = IKnockdownsPlayerData.get(entity);
                    data.setKnockedDown(packet.knockedDown);
                    data.setReviveTimeLeft(packet.reviveTimeLeft);
                }
            });
        } else if (message instanceof SynchronizeReviversS2CPacket.Add) {
            SynchronizeReviversS2CPacket.Add packet = (SynchronizeReviversS2CPacket.Add) message;
            client.addScheduledTask(() -> {
                EntityPlayer knocked = (EntityPlayer) client.world.getEntityByID(packet.knockedId);
                EntityPlayer reviver = (EntityPlayer) client.world.getEntityByID(packet.reviverId);
                if (knocked != null && reviver != null) {
                    IKnockdownsPlayerData.get(knocked).getRevivers().add(reviver);
                }
            });
        } else if (message instanceof SynchronizeReviversS2CPacket.Remove) {
            SynchronizeReviversS2CPacket.Remove packet = (SynchronizeReviversS2CPacket.Remove) message;
            client.addScheduledTask(() -> {
                EntityPlayer knocked = (EntityPlayer) client.world.getEntityByID(packet.knockedId);
                EntityPlayer reviver = (EntityPlayer) client.world.getEntityByID(packet.reviverId);
                if (knocked != null && reviver != null) {
                    IKnockdownsPlayerData.get(knocked).getRevivers().remove(reviver);
                }
            });
        } else {
            throw new IllegalStateException("Unknown packet received on the client: " + message.getClass().getName());
        }

        return null;
    }
}
