package ru.octol1ttle.knockdowns.client.event;

import java.util.List;
import net.minecraft.client.Minecraft;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraftforge.client.event.RenderGameOverlayEvent;
import net.minecraftforge.client.event.RenderWorldLastEvent;
import net.minecraftforge.fml.common.Mod;
import net.minecraftforge.fml.common.eventhandler.SubscribeEvent;
import net.minecraftforge.fml.common.gameevent.InputEvent;
import net.minecraftforge.fml.common.gameevent.TickEvent;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;
import ru.octol1ttle.knockdowns.Tags;
import ru.octol1ttle.knockdowns.client.communication.CalloutManager;
import ru.octol1ttle.knockdowns.client.communication.KnockedNotificationManager;
import ru.octol1ttle.knockdowns.client.gui.CommunicationGui;
import ru.octol1ttle.knockdowns.client.gui.KnockedNotificationGui;
import ru.octol1ttle.knockdowns.client.gui.ReviveGui;
import ru.octol1ttle.knockdowns.common.data.IKnockdownsPlayerData;
import ru.octol1ttle.knockdowns.common.network.KnockdownsNetwork;
import ru.octol1ttle.knockdowns.common.network.packets.c2s.CancelReviveC2SPacket;

@SideOnly(Side.CLIENT)
@Mod.EventBusSubscriber(value = Side.CLIENT, modid = Tags.MOD_ID)
public class KnockdownsClientEventListener {
    private static final Minecraft client = Minecraft.getMinecraft();
    private static final CommunicationGui communicationGui = new CommunicationGui();
    private static final KnockedNotificationGui notificationGui = new KnockedNotificationGui();
    private static final ReviveGui reviveGui = new ReviveGui();

    @SubscribeEvent
    public static void onTick(TickEvent.ClientTickEvent event) {
        if (event.phase == TickEvent.Phase.START) {
            return;
        }
        if (client.world == null) {
            CalloutManager.clearCallouts();
            KnockedNotificationManager.clearDatas();
            return;
        }
        CalloutManager.getCallouts().removeIf(callout -> client.world.getTotalWorldTime() - callout.getValue().getReceiveTime() > 100);
        KnockedNotificationManager.getKnockedPlayerDatas().removeIf(notification -> client.world.getTotalWorldTime() - notification.getReceiveTime() > 140);
    }

    @SubscribeEvent
    public static void onPlayerTick(TickEvent.PlayerTickEvent event) {
        List<EntityPlayer> revivers = IKnockdownsPlayerData.get(event.player).getRevivers();
        if (revivers.contains(client.player) && !event.player.equals(client.pointedEntity)) {
            KnockdownsNetwork.sendToServer(new CancelReviveC2SPacket());
            revivers.remove(client.player);
        }
    }

    @SubscribeEvent
    public static void onRenderWorldLast(RenderWorldLastEvent event) {
        notificationGui.renderNotifications(event.getPartialTicks());
        communicationGui.renderCallouts(event.getPartialTicks());
    }

    @SubscribeEvent
    public static void onRenderGameOverlay(RenderGameOverlayEvent.Chat event) {
        communicationGui.render(event.getPartialTicks(), event.getResolution());
        reviveGui.render(event.getPartialTicks(), event.getResolution());
    }

    @SubscribeEvent
    public static void onKeyInput(InputEvent.KeyInputEvent event) {
        KnockdownsKeyListener.tickKeys();
    }
}
