package ru.octol1ttle.knockdowns.common;

import java.util.ArrayList;
import java.util.List;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.entity.player.EntityPlayerMP;
import net.minecraft.init.MobEffects;
import net.minecraft.potion.Potion;
import net.minecraft.potion.PotionEffect;
import net.minecraft.server.MinecraftServer;
import ru.octol1ttle.knockdowns.common.data.IKnockdownsPlayerData;
import ru.octol1ttle.knockdowns.common.network.KnockdownsNetwork;
import ru.octol1ttle.knockdowns.common.network.packets.s2c.SynchronizePlayerDataS2CPacket;

public class KnockdownsUtils {
    public static final int INITIAL_REVIVE_TIME_LEFT = 200;
    public static final float KNOCKED_INVULNERABILITY_TICKS = 3.0f * 20.0f;
    public static final float KNOCKED_HURT_PERIOD = 1.2f;
    public static final float KNOCKED_TENACITY = 60.0f;

    public static void setKnockedInitialState(EntityPlayerMP player) {
        player.extinguish();
        player.setAir(300);
        KnockdownsUtils.clearBadPotionEffects(player);
        player.addPotionEffect(new PotionEffect(MobEffects.SLOWNESS, Integer.MAX_VALUE, 3));
    }

    public static void clearBadPotionEffects(EntityPlayerMP player) {
        List<Potion> removeList = new ArrayList<>();

        for (PotionEffect effect : player.getActivePotionEffects()) {
            if (effect.getPotion().isBadEffect()) {
                removeList.add(effect.getPotion());
            }
        }

        for (Potion potion : removeList) {
            player.removePotionEffect(potion);
        }
    }

    public static boolean allPlayersKnocked(MinecraftServer server, EntityPlayer except) {
        for (EntityPlayer player : server.getPlayerList().getPlayers()) {
            if (player.equals(except)) {
                continue;
            }
            IKnockdownsPlayerData data = IKnockdownsPlayerData.get(player);
            if (player.isEntityAlive() && !data.isKnockedDown()) {
                return false;
            }
        }
        return true;
    }

    public static void resetKnockedState(EntityPlayerMP player, IKnockdownsPlayerData data) {
        player.removePotionEffect(MobEffects.SLOWNESS);
        data.setKnockedDown(false);
        data.setReviveTimeLeft(INITIAL_REVIVE_TIME_LEFT);
        data.setTicksKnocked(0);
        data.getRevivers().clear();

        KnockdownsNetwork.sendToTrackingAndSelf(
                new SynchronizePlayerDataS2CPacket.KnockedDown(player.getEntityId(), data.isKnockedDown()),
                player
        );
    }
}
